(* file NUMERICO.M *)

(* :Title: NUMERICO *)

(* :Context: MathSource` *)

(* :Author: Jesus Rojo *)

(* :Summary: 
	Contains utilities for use of NUMERICAL package
*)

(* :Mathematica Version: 2.0 *)

BeginPackage["NUMERICO`"]

CORounding::usage="CORounding[a,prec] 
gives the real number  a  rounded off to  prec  signifiant decimal
places. Note: CORounding[3.55,2] gives 3.6 and CORounding[3.54999,2]
gives 3.5. CORounding[-3.55,2] gives -3.6 and CORounding[-3.54999,2]
gives -3.5."

CONorm1::usage="CONorm1[X,n,prec]
gives the 1-norm of the vector  X  of size  n  rounded
to  prec  significant decimal places."

CONorm2::usage="CONorm2[X,n,prec]  
gives the euclidean norm (2-norm) of the vector  X  of size  n  rounded
to  prec  significant decimal places."

CONormi::usage="CONormi[X,n,prec]
gives the maximum norm (infinity norm) of the vector  X  of size  n  rounded
to  prec  significant decimal places."

Begin["`Private`"]

CORounding[a_,prec_Integer]:=
Module[{b,b1,b2,b3,i},
If[a==0,Goto[fin],];
b=Sign[a]*N[a];
b1=RealDigits[b][[1]];
b2=RealDigits[b][[2]];
If[b1[[prec+1]]>=5,b1[[prec]]++];
b3=0;
Do[
b3=b3+10^(-i)*b1[[i]]
,{i,1,prec}];
Label[fin];
If[a==0,0.,Sign[a]*N[b3*10^b2]]
]

CONorm1[X_,n_Integer,prec_Integer]:=
Module[{s,i},
s=0.;
Do[
s=CORounding[s+CORounding[Abs[CORounding[X[[i]],prec]],prec],prec]
,{i,1,n}];
s
]

CONorm2[X_,n_Integer,prec_Integer]:=
Module[{s,i},
s=0.;
Do[
s=CORounding[s+CORounding[Abs[CORounding[X[[i]],prec]]*Abs[CORounding[X[[i]],prec]]
                                                               ,prec],prec];
,{i,1,n}];
CORounding[Sqrt[s],prec]
]

CONormi[X_,n_Integer,prec_Integer]:=
Module[{s,i},
s=0.;
Do[
s=Max[s,CORounding[Abs[X[[i]]],prec]]
,{i,1,n}];
s
]

Print[""];
Print["They have been introduced in  ** NUMERICO.M **                                  
CORounding[a,prec],
CONorm1[X,n,prec],
CONorm2[X,n,prec],
CONormi[X,n,prec]
"]

End[]

Protect[
CORounding,
CONorm1,
CONorm2,
CONormi
]

EndPackage[]


